﻿using System.Diagnostics;
using System.Threading;
using System.Threading.Tasks;
using Buddy.Coroutines;
using ff14bot;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.AClasses;
using ff14bot.Navigation;
using Clio.Utilities;
using ff14bot.RemoteWindows;
using ff14bot.NeoProfiles;
using ff14bot.BotBases;
using ff14bot.Enums;
using ff14bot.Objects;
using System.Linq;

namespace AnimusBook
{
    class Common
    {
        private const int MAX_TICKS = 100;
        private const int WAIT_TIME = 200;
        private const int MOVE_TIMEOUT_MS = 900000;
        private static string sBot = "Order Bot";

        #region Ports

        public static uint CurrentMap(uint zoneid)
        {
            switch (zoneid)
            {
                case 148:
                    return 3; // "Bentbranch Meadows";//148;

                case 154:
                    return 7; // "Fallgourd Float"; //154;

                case 152:
                    return 4; // "The Hawthorne Hut";//152;

                case 146:
                    return 20; // "Forgotten Springs";//146;

                case 145:
                    return 18; // "Camp Drybone";//145;

                case 140:
                    return 17; // "Horizon";//140;

                case 141:
                    return 53; // "Black Brush Station";//141;

                case 139:
                    return 15; // "Camp Bronze Lake";//139;

                case 134:
                    return 52; // "Summerford Farms";//134;

                case 180:
                    return 16; // "Camp Overlook";//180;

                case 135:
                    return 10; // "Moraby Drydocks";//135;

                case 138:
                    return 14; // "Aleport";//138;

                case 132:
                    return 2; // New Gridania

                case 153:
                    return 5; // Quarrymill/Camp Tranquill

                case 129:
                    return 8; // Limsa Lominsa Lower Decks

                case 130:
                    return 9; // Steps of Nald

                case 137:
                    return 11; // Costa Del Sol

                case 147:
                    return 22; // Ceruleum Processing Plant

                case 155:
                    return 23; // Camp Dragonhead

                case 156:
                    return 24; // Revenant's Toll

                case 250:
                    return 55; // Wolves Den

                case 340:
                    return 56; // Gridania Housing

                case 144:
                    return 62; // Gold Saucer

                default:
                    return 0;
            }
        }

        #endregion

        public static async Task<bool> TalkNext()
        {
            Common.LogText("Talking to Target - Click Next");
            Talk.Next();
            await Coroutine.Sleep(1500);
            return true;
        }

        public static async Task<bool> SelectYes()
        {
            Common.LogText("Talking to Target - Click Yes");
            SelectYesno.ClickYes();
            await Coroutine.Sleep(1500);
            return true;
        }
        #region SupportingFunctions

        public static async Task<bool> Dismount()
        {
            if (Core.Me.IsMounted)
            {
                Common.LogText("Dismounting Chocobo");
                Actionmanager.Dismount();
                await Coroutine.Sleep(3000);
            }
            return !Core.Me.IsMounted;
        }

        public static async Task<bool> Mount()
        {
            if (!Core.Me.IsMounted)
            {
                Common.LogText("Mounting Chocobo");
                Actionmanager.Mount();
                await Coroutine.Sleep(3000);
            }
            return Core.Me.IsMounted;
        }

        public static void ChangeBot(string bot)
        {
            Common.LogText("Changing Bot to - " + bot);
            System.Threading.Tasks.Task.Factory.StartNew(async () =>
            {
                await TreeRoot.StopGently("Preparing to switch botbase.");
                BotManager.SetCurrent(BotManager.Bots.FirstOrDefault(r => r.Name == bot));
                TreeRoot.Start();
            }); 
            return;
        }

        #endregion

        #region CharacterStatus
        public static bool Busy
        {
            get
            {
                return Core.Me.InCombat
                       || Core.Me.IsDead
                       || IsReviving
                       || Core.Me.IsCasting
                       || !TreeRoot.IsRunning;
            }
        }

        public static bool Reset
        {
            get
            {
                return Core.Me.IsDead
                    || IsReviving;
            }

        }

        public static bool InFate
        {
            get
            {
                return FateManager.WithinFate;
            }
        }


        private static bool IsReviving
        {
            get
            {
                var state = ClientGameUiRevive.ReviveState;
                return state == ReviveState.Reviving;
            }
        }

        #endregion

        #region Logging
        public static void LogText(string sText)
        {
            Logging.Write("[Animus] - " + sText);
            return;
        }

        #endregion

    }
}
