﻿using System.Diagnostics;
using System.Threading;
using System.Threading.Tasks;
using Buddy.Coroutines;
using ff14bot;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Navigation;
using Clio.Utilities;
using ff14bot.RemoteWindows;
using ff14bot.NeoProfiles;
using ff14bot.BotBases;
using ff14bot.Enums;
using ff14bot.Objects;
using ff14bot.Interfaces;
using System.Linq;

namespace AnimusBook
{
    public static class CoreBook
    {
        #region Globals
        private static bool HackFateDone = false;
        private static bool PreReqFateInProgress = false;
        private static bool AtFateLocation = false;
        #endregion

        #region AnimusBookCore

        #region TestCode

        public static async Task<bool> TestFateHook()
        {

            int CurrentFateIndex = -1;
            string sFate = "";

            AnimusBook.BookID = 1;
            CurrentFateIndex = GetNextFateTest();

            if (Common.Reset)
                CleanupGlobalsforFate();

            if (Common.Busy)
                return false;

            if (CurrentFateIndex != -1)
            {
                if (AnimusBook.iFateWaiting != -1)
                    Common.LogText("Previous Fate Complete - " + GetNextSet(AnimusBook.iFateWaiting));
                sFate = GetNextSet(CurrentFateIndex);
                FatebotSettings.Instance.ThisFateOnly = sFate;

                if (BotManager.Current.EnglishName != "Fate Bot")
                    Common.ChangeBot("Fate Bot");

                await CheckTeleport(sFate);

                //Handlers for popping fates or pre-req fates
                if (await SpecialFateHandlers(sFate) == false)
                    return false;

                //Required due to some navigation bugs that prevent it from getting to fate.
                CleanupGlobalsforFate();

                await GetReadyforFate(sFate);

                //AnimusBook.iFateWaiting = CurrentFateIndex;
                return false;
            }

            return true;

        }

        
        public static int GetNextFateTest()
        {
            return 13;
        }

        #endregion

        public static async Task<bool> StartAnimusBook()
        {

            int CurrentKillIndex = -1;
            int CurrentFateIndex = -1;
            string sFate = "";
            
            if (AnimusBook.BookID == 0)
            {
                Common.LogText("No book available.");
                return true;
            }

            AnimusBook.Book = ZodiacManager.CurrentBook;
            AnimusBook.BookID = AnimusBook.Book.BookId;

            if (Common.Reset)
                CleanupGlobalsforFate();

            if (Common.Busy)
                return false;

            CurrentKillIndex = GetNextMob();
            CurrentFateIndex = GetNextFate();

            if (CurrentKillIndex != -1)
            {
                if (AnimusBook.iKillIndex == CurrentKillIndex)
                {
                    return false;
                }
                else
                {
                    if (AnimusBook.iKillIndex != -1)
                        Common.LogText("Previous Mob Set Complete - " + GetNextSet(AnimusBook.iKillIndex));
                    if (BotManager.Current.EnglishName != "Order Bot")
                        Common.ChangeBot("Order Bot");

                    await SetNextOrder(CurrentKillIndex);
                    return false;
                }
            }

            if (CurrentFateIndex != -1)
            {

                if (AnimusBook.iFateWaiting == CurrentFateIndex)
                    return false;
                else
                {
                    if (AnimusBook.iFateWaiting != -1)
                        Common.LogText("Previous Fate Complete - " + GetNextSet(AnimusBook.iFateWaiting));
                    sFate = GetNextSet(CurrentFateIndex);
                    FatebotSettings.Instance.ThisFateOnly = sFate;

                    if (BotManager.Current.EnglishName != "Fate Bot")
                        Common.ChangeBot("Fate Bot");

                    await CheckTeleport(sFate);

                    //Handlers for popping fates or pre-req fates
                    if (await SpecialFateHandlers(sFate) == false)
                        return false;

                    //Required due to some navigation bugs that prevent it from getting to fate.
                    CleanupGlobalsforFate();

                    await GetReadyforFate(sFate);

                    AnimusBook.iFateWaiting = CurrentFateIndex;
                    return false;
                }
            }

            Common.LogText("Book Complete.");
            if (Common.CurrentMap(WorldManager.ZoneId) != 8)
            {
                CleanupGlobalsforFate();
                FatebotSettings.Instance.ThisFateOnly = sFate;
               
                if (BotManager.Current.EnglishName != "Order Bot")
                    Common.ChangeBot("Order Bot");

                NeoProfileManager.Load(AnimusBook.sBlankXML, true);

                Common.LogText("Teleporting to City");
                WorldManager.TeleportById(8);
                await Coroutine.Sleep(15000);
            }
            return true;
        }

        public static async Task<bool> SpecialFateHandlers(string sFate)
        {

            if (sFate == "The King's Justice")
            {
                await KingJusticeFate();
            }

            if (sFate == "The Enmity of My Enemy" && FateActiveCheck("The Enmity of My Enemy") == false)
            {
                FatebotSettings.Instance.ThisFateOnly = "The Enmity of My Enemy";
                if (await HandlePreReqFate(sFate, "The Enemy of My Enemy", "338.248, -4.124189, -58.41627", 1729) == false)
                    return false;

                if (FateActiveCheck("The Enemy of My Enemy"))
                    FatebotSettings.Instance.ThisFateOnly = "The Enemy of My Enemy";

                return false;
            }
            if (sFate == "The Ceruleum Road")
            {
                if (await PopFate("The Ceruleum Road", "19.69858, 3.989769, 397.3017", 1718, false) == false)
                    return false;
            }

            if (sFate == "Schism")
            {
                if (await PopFate("Schism", "117.3887, 74.2446, -249.8744", 1332, false) == false)
                    return false;
            }

            if (sFate == "Breaching North Tidegate")
            {
                if (await CompletePreReqFateFirst("Gauging North Tidegate") == false)
                    return false;
                else
                    return true;

            }

            if (sFate == "Breaching South Tidegate")
            {
                if (await CompletePreReqFateFirst("Gauging South Tidegate") == false)
                    return false;
                else
                    return true;
            }

            /*Cannot automate this right now as 2 NPC with same ID come out and Object ID keeps changing.
            if (sFate == "Tower of Power")
            {
                if (await PopFate("Tower of Power", "-549.7592, 237.8373, 359.7485", 1074028094, true) == false)
                    return false;
            }
            */
            return true;
        }


        //Required mainly for fates where Navigation is unable to get to them.
        public static void CleanupGlobalsforFate()
        {
            HackFateDone = false;
            AnimusBook.iFateWaiting = -1;
            PreReqFateInProgress = false;
            AtFateLocation = false;
            return;
        }

        public static GameObject getTargets(uint fateid)
        {

            var _target = GameObjectManager.GameObjects.Where(unit => (unit as BattleCharacter) != null && (unit as BattleCharacter).FateId == fateid && !(unit as BattleCharacter).IsDead).OrderBy(unit => unit.Distance(Core.Player.Location)).Take(100);

            Common.LogText("Searching Targets");
            var targetArray = _target as GameObject[] ?? _target.ToArray();
            if (targetArray.Length > 0) { return targetArray[0]; } else { return null; }

        }

        public static async Task<bool> SetNextOrder(int CurrentKillIndex)
        {
            string sMobSet = "";     
            
            sMobSet = GetNextSet(CurrentKillIndex);
            Common.LogText("Starting new Order Profile - " + sMobSet);

            Common.LogText("Loading Profile - " + AnimusBook.sPluginPath + AnimusBook.Books[AnimusBook.BookID] + @"\" + sMobSet);
            NeoProfileManager.Load(AnimusBook.sPluginPath + AnimusBook.Books[AnimusBook.BookID] + @"\" + sMobSet, true);
            NeoProfileManager.UpdateCurrentProfileBehavior();

            AnimusBook.iKillIndex = CurrentKillIndex;

            Common.LogText("Loaded Order Profile - " + sMobSet);

            return false;
        }

        public static int GetNextMob()
        {
            if (ZodiacManager.CurrentBook.KillTask1 < 3)
                return 1;
            if (ZodiacManager.CurrentBook.KillTask2 < 3)
                return 2;
            if (ZodiacManager.CurrentBook.KillTask3 < 3)
                return 3;
            if (ZodiacManager.CurrentBook.KillTask4 < 3)
                return 4;
            if (ZodiacManager.CurrentBook.KillTask5 < 3)
                return 5;
            if (ZodiacManager.CurrentBook.KillTask6 < 3)
                return 6;
            if (ZodiacManager.CurrentBook.KillTask7 < 3)
                return 7;
            if (ZodiacManager.CurrentBook.KillTask8 < 3)
                return 8;
            if (ZodiacManager.CurrentBook.KillTask9 < 3)
                return 9;
            if (ZodiacManager.CurrentBook.KillTask10 < 3)
                return 10;

            return -1;

        }


        public static int GetNextFate()
        {
            if (!ZodiacManager.CurrentBook.FateTask1Completed)
                return 11;
            if (!ZodiacManager.CurrentBook.FateTask2Completed)
                return 12;
            if (!ZodiacManager.CurrentBook.FateTask3Completed)
                return 13;

            return -1;
        }

        public static bool FateComplete(int FateIndex)
        {

            switch (FateIndex)
            {
                case 11:
                    return ZodiacManager.CurrentBook.FateTask1Completed;
                case 12:
                    return ZodiacManager.CurrentBook.FateTask2Completed;
                case 13:
                    return ZodiacManager.CurrentBook.FateTask3Completed;
            }

            return true;           
        }

        public static string GetNextSet(int iIndex)
        {
            string sNextBook = "";

            switch (AnimusBook.Books[AnimusBook.BookID])
            {
                case "Netherfall I":
                    sNextBook = AnimusBook.NetherfallI[iIndex];
                    break;
                case "Netherfire I":
                    sNextBook = AnimusBook.NetherfireI[iIndex];
                    break;
                case "Skyearth I":
                    sNextBook = AnimusBook.SkyearthI[iIndex];
                    break;
                case "Skyfall I":
                    sNextBook = AnimusBook.SkyfallI[iIndex];
                    break;
                case "Skyfall II":
                    sNextBook = AnimusBook.SkyfallII[iIndex];
                    break;
                case "Skyfire I":
                    sNextBook = AnimusBook.SkyfireI[iIndex];
                    break;
                case "Skyfire II":
                    sNextBook = AnimusBook.SkyfireII[iIndex];
                    break;
                case "Skywind I":
                    sNextBook = AnimusBook.SkywindI[iIndex];
                    break;
                case "Skywind II":
                    sNextBook = AnimusBook.SkywindII[iIndex];
                    break;
            }

            return sNextBook;

        }


        public static async Task<bool> GetReadyforFate(string sFate)
        {
            if (Common.InFate)
                return false;

            await CheckTeleport(sFate);

            string sLocation = AnimusBook.FateLocation[sFate];
            if (sLocation != "")
            {
                Common.LogText("Moving closer to the Fate - " + sLocation);
                Clio.Utilities.Vector3 Goto = new Clio.Utilities.Vector3(sLocation);
                await Common.Mount();
                await Movement.MoveTo(Goto);
                await Common.Dismount();
            }

            FatebotSettings.Instance.ThisFateOnly = sFate;
            FatebotSettings.Instance.MonsterSlayingEnabled = true;
            FatebotSettings.Instance.LevelCheck = false;
            FatebotSettings.Instance.IdleAction = FateIdleAction.Nothing;
            FatebotSettings.Instance.BossPercentRequired = 0;
            FatebotSettings.Instance.BossEnabled = true;
            FatebotSettings.Instance.MinLevel = 1;
            FatebotSettings.Instance.MaxLevel = 50;
            FatebotSettings.Instance.MonsterSlayingEnabled = true;

            if (FatebotSettings.Instance.BlackListedFates.Contains(sFate))
                Common.LogText("Fate - " + sFate + " is currently blacklisted.  Please stop bot, remove the fate from the blacklist in Fatebot and start the bot again.");

            return false;
        }

        public static void ClearFate()
        {
            FatebotSettings.Instance.ThisFateOnly = "";
        }

        public static async Task<bool> KingJusticeFate()
        {
            //Check to see if Isle already by looking at distance of player to center of Isle.  
            //Center of Isle to where you zone onto Isle is 211.
            Clio.Utilities.Vector3 GotoCheck = new Clio.Utilities.Vector3("-293.8705, -38.32589, 609.825");

            if (Core.Player.Distance(GotoCheck) > 350)
            {
                Clio.Utilities.Vector3 Goto = new Clio.Utilities.Vector3("316.7198, -36.325, 351.2196");

                await Movement.MoveTo(Goto);

                while (Core.Player.Distance(Goto) > 3)
                {
                    await Coroutine.Sleep(1000);
                }

                GameObjectManager.GetObjectByObjectId(4186446).Target();
                await Coroutine.Sleep(1000);
                Core.Player.CurrentTarget.Interact();
                await Coroutine.Sleep(1000);

                var lines = ff14bot.RemoteWindows.SelectIconString.Lines();
                int line = lines.Count - 2;
                SelectIconString.ClickSlot((uint)line);

                await Coroutine.Sleep(1000);
                Talk.Next();
                await Coroutine.Sleep(1000);
                SelectYesno.ClickYes();
                await Coroutine.Sleep(15000);
            }

            return true;
        }

        private static bool FateActiveCheck(string sFate)
        {
            if (FateManager.ActiveFates
                .Where(m => m.Name == sFate).Count() > 0)
                return true;
            else
                return false;
        }

        private static bool NPCIdExists(uint npcid)
        {
            return NPCIdExists(npcid, false);
        }

        private static bool NPCIdExists(uint npcid, bool useObjectID)
        {

            if (useObjectID == false)
            {
                if (GameObjectManager.GameObjects
                    .Where(m => m.NpcId == npcid && Core.Player.Distance(m.Location) < 5).Count() > 0)
                    return true;
                else
                    return false;
            }
            else
            {
                if (GameObjectManager.GameObjects
                    .Where(m => m.ObjectId == npcid && Core.Player.Distance(m.Location) < 5).Count() > 0)
                    return true;
                else
                    return false;
            }

        }

        private static bool ObjectIdExists(uint objectid)
        {
            if (GameObjectManager.GameObjects
                .Where(m => m.ObjectId == objectid).Count() > 0)
                return true;
            else
                return false;

        }

        public static async Task<bool> HandlePreReqFate(string sFate, string sPreReqFate, string sNPCLocation, uint NPCid)
        {
            //If the required fate is active or the pre-req fate is active then dont look for start NPC.
            //Once the pre-req fate is started, dont try to find the NPC again and exit.
            if (FateActiveCheck(sFate) || FateActiveCheck(sPreReqFate) || HackFateDone == true)
                return true;

            Clio.Utilities.Vector3 GotoStartNPC = new Clio.Utilities.Vector3(sNPCLocation);

            //If you get agro waiting for NPC, dont keep running back to NPC unless you exceed a distance to prevent looking like a bot.
            if (Core.Player.Distance(GotoStartNPC) > 4)
                await Movement.MoveTo(GotoStartNPC);

            if (NPCIdExists(NPCid) == false && FateActiveCheck(sFate) == false && FateActiveCheck(sPreReqFate) == false)
                return false;

            if (FateActiveCheck(sPreReqFate))
                return true;
            else
            {
                if (NPCIdExists(NPCid) == true)
                {
                    await Movement.MoveTo(GotoStartNPC);

                    GameObjectManager.GetObjectByNPCId(NPCid).Target();
                    
                    if (!Core.Me.HasTarget)
                        return false;

                    Core.Player.CurrentTarget.Interact();
                    while (NPCIdExists(NPCid))
                    {
                        await Common.TalkNext();
                        await Common.SelectYes();
                        if (NPCIdExists(NPCid) && !Core.Me.HasTarget)
                            return false;
                    }

                    Clio.Utilities.Vector3 ToFateZone = new Clio.Utilities.Vector3("317.1317, -6.623945, -2.941691");
                    await Movement.MoveTo(ToFateZone);
                    
                    //Set to true so it does not try to find the NPC again.
                    HackFateDone = true;
                    return true;
                }

            }

            return false;
        }

        public static async Task<bool> PopFate(string sFate, string sNPCLocation, uint NPCid, bool useObjectID)
        {
            //If the required fate is active or the pre-req fate is active then dont look for start NPC.
            //Once the pre-req fate is started, dont try to find the NPC again and exit.
            if (FateActiveCheck(sFate) || HackFateDone == true)
                return true;
            
            Clio.Utilities.Vector3 GotoStartNPC = new Clio.Utilities.Vector3(sNPCLocation);

            //If you get agro waiting for NPC, dont keep running back to NPC unless you exceed a distance to prevent looking like a bot.
            if (Core.Player.Distance(GotoStartNPC) > 20)
            {
                //only mount if you are really far.  Dont want to mount each time you get pulled away from Agro.  Looks Botty.
                if (Core.Player.Distance(GotoStartNPC) > 200)
                    await Common.Mount();
                await Movement.MoveTo(GotoStartNPC);
                await Common.Dismount();
            }

            if (NPCIdExists(NPCid, useObjectID) == false && FateActiveCheck(sFate) == false)
                return false;

            if (NPCIdExists(NPCid, useObjectID) == true)
            {
                await Movement.MoveTo(GotoStartNPC);
                GameObject NPCObj;

                if (useObjectID)
                    NPCObj = GameObjectManager.GetObjectByObjectId(NPCid);
                else
                    NPCObj = GameObjectManager.GetObjectByNPCId(NPCid);

                //Make sure you can target him first.  Some NPC are walking toward you but cannot be selected.
                if (!NPCObj.IsTargetable)
                    return false;

                NPCObj.Target();

                Core.Player.CurrentTarget.Interact();

                while (NPCIdExists(NPCid, useObjectID))
                {
                    await Common.TalkNext();
                    await Common.SelectYes();
                    if (NPCIdExists(NPCid, useObjectID) && !Core.Me.HasTarget)
                        return false;
                }

                //Set to true so it does not try to find the NPC again.
                HackFateDone = true;
                return true;
            }

            return false;
        }

        public static async Task<bool> CompletePreReqFateFirst(string sPreReqFate)
        {
            if (!AtFateLocation)
                await GetReadyforFate(sPreReqFate);
            
            AtFateLocation = true;

            if (FateActiveCheck(sPreReqFate))
            {
                FatebotSettings.Instance.ThisFateOnly = sPreReqFate;
                PreReqFateInProgress = true;
            }
            else if (!FateActiveCheck(sPreReqFate) && PreReqFateInProgress)
                return true;

            return false;
        }
            
        #endregion

        #region Teleport/Movement

        public static async Task<bool> CheckTeleport(string sFate)
        {

            if ((Common.CurrentMap(WorldManager.ZoneId) != AnimusBook.Fate[sFate]))
            {
                CleanupGlobalsforFate();
                await Movement.Teleport(AnimusBook.Fate[sFate]);
            }

            return true;
        }

        #endregion



    }
}
